#!/bin/env pwsh

#EXAMPLES OF USAGE
#  Take logs for two days and include nginx, auth and api logs:
#    .\LocalNETCODiagnostics.ps1 -logsFromDays 2 -nginx -auth -api
#  Take logs for two days:
#    .\LocalNETCODiagnostics.ps1 2
param (
    $logsFromDays=10,
    [switch]$nginx,
    [switch]$auth,
	[switch]$api
)

if ($IsLinux -and (-not $env:SUDO_UID))
{
    throw "This script must be run as root, aborting..."
}

$prefix = "netco_nms"
Write-Host "Generating diagnostics package for ${prefix}..."
Write-Host

if ($IsLinux ){
  $logPath = (docker volume inspect --format '{{.Mountpoint}}' ${prefix}_satel_programdata)
  if ($logPath -eq "") {
    Write-Host
    Write-Host "Has NETCO STACKS been launched from this folder?"
    [Environment]::Exit(1)
  }
}
else {
    $logPath = "$PsScriptRoot/__lastlogs"
    If(!(test-path -PathType container $logPath)){
       New-Item -Path $logPath -ItemType Directory
    }
    else {
        Remove-Item $PsScriptRoot/__lastlogs/* -Recurse -Force
    }

    $dsContainerName = docker ps --filter name=netco_nms.ds --format "{{.Names}}"
    Invoke-Expression "docker cp ${dsContainerName}:/usr/share/Satel/NETCO/. '$logPath'"
}

# Fetch logs from auth and nginx containers
$startTime = (Get-Date).AddDays(-1 * $logsFromDays).ToUniversalTime().ToString('yyyy-MM-ddTHH:mm:ss')
if ($auth) {
    $authContainerName = docker ps --filter name=netco_nms.auth --format "{{.Names}}"
    Start-Process -FilePath docker -ArgumentList @('logs', '-t', '--since', $startTime, $authContainerName) -RedirectStandardOutput "$logPath/auth_stdout.txt" -RedirectStandardError "$logPath/auth_stderr.txt" -Wait

}
if ($nginx) {
    $nginxContainerName = docker ps --filter name=netco_nms.nginx --format "{{.Names}}"
    Start-Process -FilePath docker -ArgumentList @('logs', '-t', '--since', $startTime, $nginxContainerName) -RedirectStandardOutput "$logPath/nginx_stdout.txt" -RedirectStandardError "$logPath/nginx_stderr.txt" -Wait
}
if ($api) {
    $apiContainerName = docker ps --filter name=netco_nms.api --format "{{.Names}}"
    Start-Process -FilePath docker -ArgumentList @('logs', '-t', '--since', $startTime, $apiContainerName) -RedirectStandardOutput "$logPath/api_stdout.txt" -RedirectStandardError "$logPath/api_stderr.txt" -Wait
}

$serviceContainerName = docker ps --filter name=netco_nms.logs_clean_up --format "{{.Names}}"
Start-Process -FilePath docker -ArgumentList @('logs', '-t', $serviceContainerName) -RedirectStandardOutput "$logPath/logs_clean_up_stdout.txt" -RedirectStandardError "$logPath/logs_clean_up_stderr.txt" -Wait

Add-Type -A System.IO.Compression.FileSystem
$zipFileName = "$PsScriptRoot/netco_nms_$(Get-Date -Format "yyyy-MM-dd_HHmmss")_ser.zip"
$zip = [System.IO.Compression.ZipFile]::Open($zipFileName, 'update')
$compressionLevel = [System.IO.Compression.CompressionLevel]::Optimal

Set-Location -Path $logPath
if ($IsLinux ){
    df -h > disk_usage.txt
    free -m > memory_usage.txt
    uptime > uptime.txt
    docker stats --no-stream > docker_stats.txt
}
else {
	# Disk usage equivalent to df -h
    $diskUsageOutput = Get-PSDrive | Out-String
    $diskUsageOutput | Out-File -FilePath "$logPath\disk_usage.txt" -Encoding UTF8

    # Memory usage equivalent to free -m
    $memoryUsageOutput = Get-CimInstance -ClassName Win32_OperatingSystem | Select-Object TotalVisibleMemorySize,FreePhysicalMemory | Out-String
    $memoryUsageOutput | Out-File -FilePath "$logPath\memory_usage.txt" -Encoding UTF8

    # Uptime equivalent to uptime
    $uptimeOutput = Get-CimInstance -ClassName Win32_OperatingSystem | Select-Object LastBootUpTime | Out-String
    $uptimeOutput | Out-File -FilePath "$logPath\uptime.txt" -Encoding UTF8
    
    # Docker stats
    docker stats --no-stream | Out-File -FilePath "$logPath\docker_stats.txt"
}

$cutoffDate = (Get-Date).AddDays(-$logsFromDays)
Get-ChildItem -Path . -Recurse -Include ('*.log*','package_config*.json', '*.csv*', '*.txt') |
    ForEach-Object {
        $takeFile = $false
        if ($_.Name -match '^trace_(\d{4}-\d{2}-\d{2})') {
            $fileDate = [datetime]::ParseExact($matches[1], 'yyyy-MM-dd', $null)
            $takeFile = $fileDate -ge $cutoffDate
            if ($fileDate -ge $cutoffDate) {
                $logFile = $_.FullName
                $logPrefixLength = $logPath.length
                $logZipPath = $logFile.Remove(0, $logPrefixLength)
                $logZipPath = $logZipPath.TrimStart("/")
                $logZipPath = $logZipPath.TrimStart("\")
                Write-Host "  $($logZipPath)"
                [System.IO.Compression.ZipFileExtensions]::CreateEntryFromFile($zip, $logFile, $logZipPath, $compressionLevel) 1>$null
            }
        } elseif ($_.Name -match '^events_') {
            $takeFile = $_.CreationTime -ge $cutoffDate
        } elseif ($_.Name -match 'package_config.*\.json') {
            $takeFile = $true
		} elseif ($_.Name -match '.*\.txt') {
            $takeFile = $true
        }
        if ($takeFile) {
            $logFile = $_.FullName
            $logPrefixLength = $logPath.length
            $logZipPath = $logFile.Remove(0, $logPrefixLength)
            $logZipPath = $logZipPath.TrimStart("/")
            $logZipPath = $logZipPath.TrimStart("\")
            Write-Host "  $($logZipPath)"
            [System.IO.Compression.ZipFileExtensions]::CreateEntryFromFile($zip, $logFile, $logZipPath, $compressionLevel) 1>$null
        }
 }

$zip.Dispose()
Write-Host
Write-Host "Generated file: ${zipFileName}"
Set-Location -Path $PsScriptRoot
